@@ -38,6 +38,15 @@
 
 #include "tracker-db-interface-sqlite.h"
 
+#define UNKNOWN_STATUS 0.5
+
+typedef struct {
+	TrackerDBStatement *head;
+	TrackerDBStatement *tail;
+	guint size;
+	guint max;
+} TrackerDBStatementLru;
+
 struct TrackerDBInterface {
 	GObject parent_instance;
 
@@ -59,6 +68,13 @@ struct TrackerDBInterface {
 	/* async operation pending */
 	gboolean pending;
 	GAsyncReadyCallback outstanding_callback;
+
+	TrackerDBStatementLru select_stmt_lru;
+	TrackerDBStatementLru update_stmt_lru;
+
+	TrackerBusyCallback busy_callback;
+	gpointer busy_user_data;
+	gchar *busy_status;
 };
 
 struct TrackerDBInterfaceClass {
@@ -72,6 +88,8 @@ struct TrackerDBCursor {
 	gboolean finished;
 	TrackerPropertyType *types;
 	gint n_types;
+	gchar **variable_names;
+	gint n_variable_names;
 };
 
 struct TrackerDBCursorClass {
@@ -83,6 +101,8 @@ struct TrackerDBStatement {
 	TrackerDBInterface *db_interface;
 	sqlite3_stmt *stmt;
 	gboolean stmt_is_sunk;
+	TrackerDBStatement *next;
+	TrackerDBStatement *prev;
 };
 
 struct TrackerDBStatementClass {
@@ -95,7 +115,9 @@ static void                tracker_db_statement_sqlite_reset (TrackerDBStatement
 static TrackerDBCursor    *tracker_db_cursor_sqlite_new      (sqlite3_stmt         *sqlite_stmt,
                                                               TrackerDBStatement   *ref_stmt,
                                                               TrackerPropertyType  *types,
-                                                              gint                  n_types);
+                                                              gint                  n_types,
+                                                              const gchar         **variable_names,
+                                                              gint                  n_variable_names);
 static gboolean            tracker_db_cursor_get_boolean     (TrackerSparqlCursor  *cursor,
                                                               guint                 column);
 static gboolean            db_cursor_iter_next               (TrackerDBCursor      *cursor,
@@ -494,6 +516,13 @@ static int
 check_interrupt (void *user_data)
 {
 	TrackerDBInterface *db_interface = user_data;
+
+	if (db_interface->busy_callback) {
+		db_interface->busy_callback (db_interface->busy_status,
+		                             UNKNOWN_STATUS, /* No idea to get the status from SQLite */
+		                             db_interface->busy_user_data);
+	}
+
 	return g_cancellable_is_cancelled (db_interface->cancellable) ? 1 : 0;
 }
 
@@ -703,6 +732,7 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 	g_message ("Closed sqlite3 database:'%s'", db_interface->filename);
 
 	g_free (db_interface->filename);
+	g_free (db_interface->busy_status);
 
 	tracker_collation_shutdown (db_interface->collator);
 
@@ -811,13 +841,55 @@ add_row (TrackerDBResultSet *result_set,
 	}
 }
 
+void
+tracker_db_interface_set_max_stmt_cache_size (TrackerDBInterface         *db_interface,
+                                              TrackerDBStatementCacheType cache_type,
+                                              guint                       max_size)
+{
+	TrackerDBStatementLru *stmt_lru;
+
+	if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE) {
+		stmt_lru = &db_interface->update_stmt_lru;
+	} else if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT) {
+		stmt_lru = &db_interface->select_stmt_lru;
+	} else {
+		return;
+	}
+
+	/* Must be larger than 2 to make sense (to have a tail and head) */
+	if (max_size > 2) {
+		stmt_lru->max = max_size;
+	} else {
+		stmt_lru->max = 3;
+	}
+}
+
+void
+tracker_db_interface_set_busy_handler (TrackerDBInterface  *db_interface,
+                                       TrackerBusyCallback  busy_callback,
+                                       const gchar         *busy_status,
+                                       gpointer             busy_user_data)
+{
+	g_return_if_fail (TRACKER_IS_DB_INTERFACE (db_interface));
+	db_interface->busy_callback = busy_callback;
+	db_interface->busy_user_data = busy_user_data;
+	g_free (db_interface->busy_status);
+
+	if (busy_status) {
+		db_interface->busy_status = g_strdup (busy_status);
+	} else {
+		db_interface->busy_status = NULL;
+	}
+}
 
 TrackerDBStatement *
-tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
-                                       GError             **error,
-                                       const gchar         *query,
+tracker_db_interface_create_statement (TrackerDBInterface           *db_interface,
+                                       TrackerDBStatementCacheType   cache_type,
+                                       GError                      **error,
+                                       const gchar                  *query,
                                        ...)
 {
+	TrackerDBStatementLru *stmt_lru = NULL;
 	TrackerDBStatement *stmt;
 	va_list args;
 	gchar *full_query;
@@ -829,10 +901,35 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 	full_query = g_strdup_vprintf (query, args);
 	va_end (args);
 
-	stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
+	/* There are three kinds of queries: 
+	  * a) Cached queries: SELECT and UPDATE ones (cache_type)
+	  * b) Non-Cached queries: NONE ones (cache_type)
+	  * c) Forced Non-Cached: in case of a stmt being already in use, we can't
+	  *    reuse it (you can't use two different loops on a sqlite3_stmt, of
+	  *    course). This happens with recursive uses of a cursor, for example */
 
-	if (stmt && stmt->stmt_is_sunk) {
-		/* prepared statement is still in use, create new one */
+	if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+		stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
+
+		if (stmt && stmt->stmt_is_sunk) {
+			/* c) Forced non-cached
+			 * prepared statement is still in use, create new uncached one */
+			stmt = NULL;
+			/* Make sure to set cache_type here, to ensure the right flow in the
+			 * LRU-cache below */
+			cache_type = TRACKER_DB_STATEMENT_CACHE_TYPE_NONE;
+		}
+
+		if (cache_type == TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE) {
+			/* a) Cached */
+			stmt_lru = &db_interface->update_stmt_lru;
+		} else {
+			/* a) Cached */
+			stmt_lru = &db_interface->select_stmt_lru;
+		}
+
+	} else {
+		/* b) Non-Cached */
 		stmt = NULL;
 	}
 
@@ -866,19 +963,91 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 
 		stmt = tracker_db_statement_sqlite_new (db_interface, sqlite_stmt);
 
-		/* use replace instead of insert to make sure we store the string that
-		   belongs to the right sqlite statement to ensure the lifetime of the string
-		   matches the statement */
-		g_hash_table_replace (db_interface->dynamic_statements,
-		                      (gpointer) sqlite3_sql (sqlite_stmt),
-		                      stmt);
+		if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+			/* use replace instead of insert to make sure we store the string that
+			   belongs to the right sqlite statement to ensure the lifetime of the string
+			   matches the statement */
+			g_hash_table_replace (db_interface->dynamic_statements,
+			                      (gpointer) sqlite3_sql (sqlite_stmt),
+			                      stmt);
+
+			/* So the ring looks a bit like this: *
+			 *                                    *
+			 *    .--tail  .--head                *
+			 *    |        |                      *
+			 *  [p-n] -> [p-n] -> [p-n] -> [p-n]  *
+			 *    ^                          |    *
+			 *    `- [n-p] <- [n-p] <--------'    *
+			 *                                    */
+
+			if (stmt_lru->size >= stmt_lru->max) {
+				TrackerDBStatement *new_head;
+
+				/* We reached max-size of the LRU stmt cache. Destroy current
+				 * least recently used (stmt_lru.head) and fix the ring. For
+				 * that we take out the current head, and close the ring.
+				 * Then we assign head->next as new head. */
+
+				new_head = stmt_lru->head->next;
+				g_hash_table_remove (db_interface->dynamic_statements,
+				                     (gpointer) sqlite3_sql (stmt_lru->head->stmt));
+				stmt_lru->size--;
+				stmt_lru->head = new_head;
+			} else {
+				if (stmt_lru->size == 0) {
+					stmt_lru->head = stmt;
+					stmt_lru->tail = stmt;
+				}
+			}
+
+			/* Set the current stmt (which is always new here) as the new tail
+			 * (new most recent used). We insert current stmt between head and
+			 * current tail, and we set tail to current stmt. */
+
+			stmt_lru->size++;
+			stmt->next = stmt_lru->head;
+			stmt_lru->head->prev = stmt;
+
+			stmt_lru->tail->next = stmt;
+			stmt->prev = stmt_lru->tail;
+			stmt_lru->tail = stmt;
+		}
 	} else {
 		tracker_db_statement_sqlite_reset (stmt);
+
+		if (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) {
+			if (stmt == stmt_lru->head) {
+
+				/* Current stmt is least recently used, shift head and tail
+				 * of the ring to efficiently make it most recently used. */
+
+				stmt_lru->head = stmt_lru->head->next;
+				stmt_lru->tail = stmt_lru->tail->next;
+			} else if (stmt != stmt_lru->tail) {
+
+				/* Current statement isn't most recently used, make it most
+				 * recently used now (less efficient way than above). */
+
+				/* Take stmt out of the list and close the ring */
+				stmt->prev->next = stmt->next;
+				stmt->next->prev = stmt->prev;
+
+				/* Put stmt as tail (most recent used) */
+				stmt->next = stmt_lru->head;
+				stmt_lru->head->prev = stmt;
+				stmt->prev = stmt_lru->tail;
+				stmt_lru->tail->next = stmt;
+				stmt_lru->tail = stmt;
+			}
+
+			/* if (stmt == tail), it's already the most recently used in the
+			 * ring, so in this case we do nothing of course */
+		}
 	}
 
 	g_free (full_query);
 
-	return g_object_ref (stmt);
+	return (cache_type != TRACKER_DB_STATEMENT_CACHE_TYPE_NONE) ? g_object_ref (stmt) : stmt;
 }
 
 static TrackerDBResultSet *
@@ -1092,11 +1261,17 @@ static void
 tracker_db_cursor_finalize (GObject *object)
 {
 	TrackerDBCursor *cursor;
+	int i;
 
 	cursor = TRACKER_DB_CURSOR (object);
 
 	g_free (cursor->types);
 
+	for (i = 0; i < cursor->n_variable_names; i++) {
+		g_free (cursor->variable_names[i]);
+	}
+	g_free (cursor->variable_names);
+
 	if (cursor->ref_stmt) {
 		cursor->ref_stmt->stmt_is_sunk = FALSE;
 		tracker_db_statement_sqlite_reset (cursor->ref_stmt);
@@ -1198,7 +1373,9 @@ static TrackerDBCursor *
 tracker_db_cursor_sqlite_new (sqlite3_stmt        *sqlite_stmt,
                               TrackerDBStatement  *ref_stmt,
                               TrackerPropertyType *types,
-                              gint                 n_types)
+                              gint                 n_types,
+                              const gchar        **variable_names,
+                              gint                 n_variable_names)
 {
 	TrackerDBCursor *cursor;
 
@@ -1224,6 +1401,16 @@ tracker_db_cursor_sqlite_new (sqlite3_stmt        *sqlite_stmt,
 		}
 	}
 
+	if (variable_names) {
+		gint i;
+
+		cursor->variable_names = g_new (gchar *, n_variable_names);
+		cursor->n_variable_names = n_variable_names;
+		for (i = 0; i < n_variable_names; i++) {
+			cursor->variable_names[i] = g_strdup (variable_names[i]);
+		}
+	}
+
 	return cursor;
 }
 
@@ -1428,7 +1615,11 @@ const gchar*
 tracker_db_cursor_get_variable_name (TrackerDBCursor *cursor,
                                      guint            column)
 {
-	return sqlite3_column_name (cursor->stmt, column);
+	if (column < cursor->n_variable_names) {
+		return cursor->variable_names[column];
+	} else {
+		return sqlite3_column_name (cursor->stmt, column);
+	}
 }
 
 const gchar*
@@ -1461,18 +1652,20 @@ tracker_db_statement_start_cursor (TrackerDBStatement  *stmt,
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, NULL, 0, NULL, 0);
 }
 
 TrackerDBCursor *
 tracker_db_statement_start_sparql_cursor (TrackerDBStatement   *stmt,
                                           TrackerPropertyType  *types,
                                           gint                  n_types,
+                                          const gchar         **variable_names,
+                                          gint                  n_variable_names,
                                           GError              **error)
 {
 	g_return_val_if_fail (!stmt->stmt_is_sunk, NULL);
 
-	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types);
+	return tracker_db_cursor_sqlite_new (stmt->stmt, stmt, types, n_types, variable_names, n_variable_names);
 }
 
 static void
