@@ -26,9 +26,6 @@
 #include <stdlib.h>
 #include <math.h>
 
-/* I don't know why, but this prototype ain't in my math.h */
-long long int llroundl(long double x);
-
 #include <glib.h>
 #include <glib/gstdio.h>
 
@@ -330,7 +327,7 @@ add_time_gst_tag (TrackerSparqlBuilder   *metadata,
 	if (ret) {
 		gint64 duration;
 
-		duration = llroundl ((long double) n / (long double) GST_SECOND);
+		duration = (n + (GST_SECOND / 2)) / GST_SECOND;
 
 		tracker_sparql_builder_predicate (metadata, key);
 		tracker_sparql_builder_object_int64 (metadata, duration);
@@ -499,7 +496,7 @@ extract_metadata (MetadataExtractor      *extractor,
                   gchar                 **scount)
 {
 	const gchar *temp;
-	gchar *s, *make = NULL, *model = NULL, *manuf = NULL, *make_model = NULL;
+	gchar *s, *make = NULL, *model = NULL, *manuf = NULL;
 	gboolean ret;
 	gint count;
 	gboolean needs_audio = FALSE;
@@ -764,26 +761,31 @@ extract_metadata (MetadataExtractor      *extractor,
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_DEVICE_MAKE, &make);
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_DEVICE_MANUFACTURER, &manuf);
 
-		if (make && model && manuf) {
-			make_model = tracker_merge_const (" ", 3, manuf, make, model);
-		} else if (make && model) {
-			make_model = tracker_merge_const (" ", 2, make, model);
-		} else if (make && manuf) {
-			make_model = tracker_merge_const (" ", 2, manuf, make);
-		} else if (model && manuf) {
-			make_model = tracker_merge_const (" ", 2, manuf, model);
-		} else if (model) {
-			make_model = g_strdup (model);
-		} else if (make) {
-			make_model = g_strdup (make);
-		} else if (manuf) {
-			make_model = g_strdup (manuf);
-		}
+		if (make || model || manuf) {
+			gchar *equip_uri;
 
-		if (make_model) {
-			tracker_sparql_builder_predicate (metadata, "nfo:device");
-			tracker_sparql_builder_object_unvalidated (metadata, make_model);
-			g_free (make_model);
+			equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+			                                              manuf ? manuf : make ? make : "",
+			                                              model ? model : "");
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+			if (make || manuf) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+				tracker_sparql_builder_object_unvalidated (preupdate, manuf ? manuf : make);
+			}
+			if (model) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:model");
+				tracker_sparql_builder_object_unvalidated (preupdate, model);
+			}
+			tracker_sparql_builder_insert_close (preupdate);
+
+			tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+			tracker_sparql_builder_object_iri (metadata, equip_uri);
+			g_free (equip_uri);
 		}
 
 		g_free (make);
