@@ -637,18 +637,29 @@ extract_pdf (const gchar          *uri,
 			}
 
 			if (xd->make || xd->model) {
-				gchar *camera;
+				gchar *equip_uri;
 
-				if ((xd->make == NULL || xd->model == NULL) ||
-					(xd->make && xd->model && strstr (xd->model, xd->make) == NULL)) {
-					camera = tracker_merge_const (" ", 2, xd->make, xd->model);
-				} else {
-					camera = g_strdup (xd->model);
+				equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+				                                              xd->make ? xd->make : "",
+				                                              xd->model ? xd->model : "");
+
+				tracker_sparql_builder_insert_open (preupdate, NULL);
+				tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+				tracker_sparql_builder_predicate (preupdate, "a");
+				tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+				if (xd->make) {
+					tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->make);
 				}
-
-				tracker_sparql_builder_predicate (metadata, "nfo:device");
-				tracker_sparql_builder_object_unvalidated (metadata, camera);
-				g_free (camera);
+				if (xd->model) {
+					tracker_sparql_builder_predicate (preupdate, "nfo:model");
+					tracker_sparql_builder_object_unvalidated (preupdate, xd->model);
+				}
+				tracker_sparql_builder_insert_close (preupdate);
+				tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+				tracker_sparql_builder_object_iri (metadata, equip_uri);
+				g_free (equip_uri);
 			}
 
 			if (xd->orientation) {
