@@ -38,7 +38,8 @@ typedef enum {
 } TagType;
 
 typedef struct {
-	gchar *camera;
+	const gchar *make;
+	const gchar *model;
 	const gchar *title;
 	const gchar *orientation;
 	const gchar *copyright;
@@ -354,16 +355,6 @@ extract_tiff (const gchar          *uri,
 	TIFFClose (image);
 	g_free (filename);
 
-	md.camera = tracker_merge_const (" ", 2, xd->make, xd->model);
-
-	if (!md.camera) {
-		md.camera = tracker_merge_const (" ", 2, td.make, td.model);
-
-		if (!md.camera) {
-			md.camera = tracker_merge_const (" ", 2, ed->make, ed->model);
-		}
-	}
-
 	md.title = tracker_coalesce_strip (5, xd->title, xd->pdf_title, td.title, ed->document_name, xd->title2);
 	md.orientation = tracker_coalesce_strip (4, xd->orientation, td.orientation, ed->orientation, id->image_orientation);
 	md.copyright = tracker_coalesce_strip (4, xd->rights, td.copyright, ed->copyright, id->copyright_notice);
@@ -384,6 +375,8 @@ extract_tiff (const gchar          *uri,
 	md.creator = tracker_coalesce_strip (3, xd->creator, id->byline, id->credit);
 	md.x_dimension = tracker_coalesce_strip (2, td.width, ed->x_dimension);
 	md.y_dimension = tracker_coalesce_strip (2, td.length, ed->y_dimension);
+	md.make = tracker_coalesce_strip (3, xd->make, td.make, ed->make);
+	md.model = tracker_coalesce_strip (3, xd->model, td.model, ed->model);
 
 	keywords = g_ptr_array_new ();
 
@@ -542,9 +535,30 @@ extract_tiff (const gchar          *uri,
 	}
 	g_ptr_array_free (keywords, TRUE);
 
-	if (md.camera) {
-		tracker_sparql_builder_predicate (metadata, "nfo:device");
-		tracker_sparql_builder_object_unvalidated (metadata, md.camera);
+	if (md.make || md.model) {
+		gchar *equip_uri;
+
+		equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+		                                              md.make ? md.make : "",
+		                                              md.model ? md.model : "");
+
+		tracker_sparql_builder_insert_open (preupdate, NULL);
+		tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+		tracker_sparql_builder_predicate (preupdate, "a");
+		tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+		if (md.make) {
+			tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+			tracker_sparql_builder_object_unvalidated (preupdate, md.make);
+		}
+		if (md.model) {
+			tracker_sparql_builder_predicate (preupdate, "nfo:model");
+			tracker_sparql_builder_object_unvalidated (preupdate, md.model);
+		}
+		tracker_sparql_builder_insert_close (preupdate);
+		tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+		tracker_sparql_builder_object_iri (metadata, equip_uri);
+		g_free (equip_uri);
 	}
 
 	if (md.title) {
@@ -678,7 +692,6 @@ extract_tiff (const gchar          *uri,
 		tracker_sparql_builder_object_double (metadata, value);
 	}
 
-	g_free (md.camera);
 	tiff_data_free (&td);
 	tracker_exif_free (ed);
 	tracker_xmp_free (xd);
